define([ "jquery", "knockout", "text!templates/entrytemplate.html", "domReady",
		"knockout_validation" ], function($, ko, entryHTML, domReady,
		validation) {

	var emptyString = "";
	var astMsg = "Required";
	var invMsg = "Invalid";
	var emailRegX = /^[a-z0-9!#$%&'*+\/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+\/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$/i; // thanks: http://stackoverflow.com/a/1373724/275501
	var phoneRegX = /(?:\d{3}|\(\d{3}\))([-\/\.])\d{3}\1\d{4}/;

	var directoryViewModel= function() {
		var self = this;
		self.entry = {
			firstName : ko.observable().extend({
				required : {
					params : true,
					message : astMsg
				}
			}),
			lastName : ko.observable().extend({
				required : {
					params : true,
					message : astMsg
				}
			}),
			title : ko.observable().extend({
				required : {
					params : true,
					message : astMsg
				}
			}),
			phone : ko.observable().extend({
				customRegEx : {
					regX : phoneRegX,
					blankMsg : astMsg,
					invalidMsg : invMsg
				}
			}),
			email : ko.observable().extend({
				customRegEx : {
					regX : emailRegX,
					blankMsg : astMsg,
					invalidMsg : invMsg
				}
			})
		};

		self.entries = ko.observableArray();

		self.isFocused = ko.observable(false);

		self.addEntry = function(e) {

			var newEntry = {
				firstName : self.entry.firstName(),
				lastName : self.entry.lastName(),
				title : self.entry.title(),
				phone : self.entry.phone(),
				email : self.entry.email()
			};

			self.entries.push(newEntry);
			self.clearForm();
			self.isFocused(true);
		};

		self.isValidForm = ko.computed(function() {
			if (self.entry.firstName.isValid() && self.entry.lastName.isValid()
					&& self.entry.title.isValid() && self.entry.phone.isValid()
					&& self.entry.email.isValid()) {
				return true;
			} else {
				return false;
			}
		}, this);

		self.removeEntry = function(entry) {
			self.entries.remove(entry);
		};

		self.clearForm = function() {
			self.entry.firstName(emptyString);
			self.entry.lastName(emptyString);
			self.entry.title(emptyString);
			self.entry.phone(emptyString);
			self.entry.email(emptyString);
		};

	}

	function getValidationConfig() {
		return {
			insertMessages : false
		};
	}

	function createCustomValidationRule() {
		ko.validation.rules["customRegEx"] = {
			validator : function(userInput, ruleObj) {
				if (!userInput || userInput.length == 0) {
					this.message = ruleObj.blankMsg;
					return false;
				}
				if (!ruleObj.regX.test(userInput)) {
					this.message = ruleObj.invalidMsg;
					return false;
				}
				return true;
			},
		};

		ko.validation.registerExtenders();
	}

	function init() {
		$("#entryList").html(entryHTML);
		createCustomValidationRule();
		domReady(function() {
			ko.applyBindingsWithValidation(directoryViewModel,
					$("#directoryContent")[0], getValidationConfig());
		});
	}
	;

	return {
		init : init
	};

});