define([ "jquery", "underscore", "backbone"], function($, _, Backbone) {

	var TipFormModel = Backbone.Model.extend({
		urlRoot: "/mockrequest",
		defaults : {
			tipPercent: 0,
    		billTotal: 0.00,
    		newBillTotal: 0.00,
    		tipAmount: 0.00,
    		tipRating: "N/A"
    	},
    	initialize: function() {
    		this.listenTo(this, "change:billTotal", this.handleChange);
    		this.listenTo(this, "change:tipPercent", this.handleChange);
    	},
    	handleChange : function() {
			// reset calculated values
			this.set({tipAmount : 0.00});
			this.set({newBillTotal : 0.00});
			
			// calculate
			this.calcTipAmount(this.get("billTotal"), this.get("tipPercent"));
			this.calcNewBillTotal(this.get("billTotal"), this.get("tipAmount"));
			this.applyTipRating(this.get("tipPercent"));				
		},	 
		calcTipAmount : function(billTotal, tipPercent) {
			var tipAmount = billTotal * (tipPercent / 100);
			this.set({tipAmount : tipAmount});
		},
		calcNewBillTotal : function(billTotal, tipAmount) {
			var newTotal = billTotal + tipAmount;
			this.set({newBillTotal : newTotal});
		},
		applyTipRating : function(roundedTip) {
			var tipRating= this.defaults.tipRating;
			if ($.isNumeric(roundedTip)) {
				if (roundedTip > 0 && roundedTip < 15) {
					tipRating = "So so";
				} else if (roundedTip == 15) {
					tipRating = "Standard";
				} else if (roundedTip > 15 && roundedTip <= 20) {
					tipRating = "Great!";
				} else if (roundedTip > 20 && roundedTip <= 50) {
					tipRating = "Super!";
				} else if (roundedTip > 50) {
					tipRating = "WOW!";
				}
			}
			this.set({tipRating : tipRating});
		},
		setBillTotal : function(billTotal) {
			if($.isNumeric(billTotal)){
				this.set({billTotal : Number(billTotal)});
			} else {
				this.set({billTotal : null});
			}
		},
		roundTipPercent : function(tipPercent) {
			if($.isNumeric(tipPercent)){
				this.set({tipPercent : Math.round(Number(tipPercent))});
			} else {
				this.set({tipPercent : null});
			}
			return this.get("tipPercent");
		}	
	});

	return new TipFormModel({});
});