define([ "jquery", "sinon", "models/tipForm" ], function($, sinon, TipForm) {

	function addTests() {
		
		module("Tip Form Model Tests", {
			beforeEach: function() {
		        this.tipFormModel = TipForm;
		        this.server = sinon.fakeServer.create();
		    },
		    afterEach: function() {
		    	this.server.restore();
		    }
		});
		
		test("Tip Calculations", function() {
			this.tipFormModel.calcTipAmount(10.00, 15);
			strictEqual(this.tipFormModel.get("tipAmount"), 1.50, "When the tip is 15% and the bill is $10, the tip amount should be $1.50");
			this.tipFormModel.calcNewBillTotal(10.00, 1.50);
			strictEqual(this.tipFormModel.get("newBillTotal"), 11.50, "When the bill is is $10 and the tip amount is $1.50, the new bill total should be $11.50");
		});	
		
		test("Tip Ratings", function() {
			this.tipFormModel.applyTipRating(5);
			strictEqual(this.tipFormModel.get("tipRating"), "So so", "When the tip amount is below $15, the rating should be So So");
			this.tipFormModel.applyTipRating(15);
			strictEqual(this.tipFormModel.get("tipRating"), "Standard", "When the tip amount equals $15, the rating should be Standard");
			this.tipFormModel.applyTipRating(20);
			strictEqual(this.tipFormModel.get("tipRating"), "Great!", "When the tip amount is between $15 and $20 (inclusive), the rating should be Great!");
			this.tipFormModel.applyTipRating(50);
			strictEqual(this.tipFormModel.get("tipRating"), "Super!", "When the tip amount is between $20 and $50 (inclusive), the rating should be Super!");
			this.tipFormModel.applyTipRating(60);
			strictEqual(this.tipFormModel.get("tipRating"), "WOW!", "When the tip amount is greater than $50, the rating should be Wow!");
		});		
		
		test("Save Request", function() {
	        this.server.respondWith([200, {"Content-Type":"text/html"}, "{'save_status':'saved'}"]);
	        this.tipFormModel.save();
	        this.server.respond();	
			equal(this.tipFormModel.urlRoot, "/mockrequest", "When the model is saved, the URL should be /mockrequest");
			equal(this.server.requests.length, 1, "When the model is saved, the request length should be 1");
			equal(this.server.requests[0].requestBody, JSON.stringify(this.tipFormModel.attributes), "When the model is saved, the request body should equal the model attributes");
		});		
		
	}
	;


	return {
		addTests : addTests
	};

});
