define([ "jquery", "knockout", "text!templates/tipDatatemplate.html",
		"domReady", "knockout_validation", "utils/tipCalculator" ], function($,
		ko, templateHTML, domReady, validation, tipCalculator) {

	function tipCalculatorViewModel() {
		var self = this;
		
		// form
		self.tipForm = {
			billTotal : ko.observable().extend({
				number : {
					params : true
				}
			}),
			tipPercent : ko.observable().extend({
				number : {
					params : true
				}
			})
		};

		// validation
		self.isValidForm = ko.computed(function() {
			if (self.tipForm.billTotal.isValid()
					&& self.tipForm.tipPercent.isValid()) {
				return true;
			} else {
				return false;
			}
		}, this);

		// used for display
		self.amt = ko.observable(tipCalculator.getDefaultData().amt);
		self.total = ko.observable(tipCalculator.getDefaultData().total);
		self.tipRating = ko.observable(tipCalculator.getDefaultRating());
		
		// calculations only on submit
		self.calculateTip = function() {
			if (self.isValidForm()) {
				self.tipForm.tipPercent(tipCalculator.roundTip(self.tipForm.tipPercent()));
				self.amt(tipCalculator.calcTipAmount(self.tipForm.tipPercent(), self.tipForm.billTotal()));
				self.total(tipCalculator.calcNewBillTotal(self.tipForm.billTotal(), self.amt()));
				self.tipRating(tipCalculator.getTipRating(self.tipForm.tipPercent()));
			}
		};
		
		// default form state
		self.noTip = function() {
			self.tipForm.billTotal("");
			self.tipForm.tipPercent("");
			self.resetTipData();
		};
		
		// default display state
		self.resetTipData = function(){
			var defaultData = tipCalculator.getDefaultData();
			self.amt(defaultData.amt);
			self.total(defaultData.total);
			self.tipRating(tipCalculator.getDefaultRating());
		};
		

	};

	function getValidationConfig() {
		return {
			insertMessages : false
		};
	}
	;
	
	function init() {
		$("#tip").html(templateHTML);
		domReady(function() {
			ko.applyBindingsWithValidation(tipCalculatorViewModel,
					$("#appShell")[0], getValidationConfig());
		});
	}
	;

	return {
		init : init,
		tipCalculatorViewModel : tipCalculatorViewModel
	};

});