angular.module("shoppingCart.controllers", ["shoppingCart.services", "data.appData", "messaging.services"])

.controller("viewCartController", [ "$q", "$scope", "$stateParams", "shoppingCartSvc", "cartData", "messageSvc", function($q, $scope, $stateParams, shoppingCartSvc, cartData, messageSvc){

	// fetch contents of cart
	var promise = shoppingCartSvc.getCart();
	handleResponse(promise, null);

	$scope.updateQuantity = function() {
		// create an object to submit to server
		var uCart = shoppingCartSvc.createCartForUpdate($scope.cart);
		
		var promise = shoppingCartSvc.updateCart(uCart);
		handleResponse(promise, "Cart updated!");	
	};
	
	$scope.removeItem = function(productId) {
		var promise = shoppingCartSvc.removeAllProductItems(productId);
		handleResponse(promise, "Product removed!");				
	};
	
	function handleResponse(promise, userMsg){
		// recalculate the cart totals
		promise.then(function(cartReturned) {
			return shoppingCartSvc.calculateTotalCartCosts(cartReturned);
		})		
		// update view
		.then(function(recalculatedCart) {
			replaceCartInView(recalculatedCart);
		})
		// notice to user
		.then(function() {
			messageSvc.displayMsg(userMsg);
		})	
		// deal with errors
		["catch"](function(errorResult) {
			messageSvc.displayError(errorResult);
		});
	};

	
	function replaceCartInView(recalculatedCart){
		$scope.totalAmt = recalculatedCart.total;
		$scope.cart = recalculatedCart;
	};

}]);