angular.module("shoppingCart.services", [ "ngResource", "data.appData", "messaging.services", "pricing.services" ])

.factory("shoppingCartSvc", ["$resource", "messageSvc", "cartData", "pricingSvc", function($resource, messageSvc, cartData, pricingSvc) {

	var Cart = $resource("controllers/shopping/carts", {
		cartId : "@cartId",
		productId : "@productId"
	}, {
		// cart methods
		getCart : {
			method : "GET",
			url : "controllers/shopping/carts/:cartId"
		},
		updateCart : {
			method : "PUT",
			url : "controllers/shopping/carts/:cartId"
		},
		// item-level methods
		addProductItem : {
			method : "POST",
			url : "controllers/shopping/carts/:cartId/products/:productId"
		},
		removeAllProductItems : {
			method : "DELETE",
			url : "controllers/shopping/carts/:cartId/products/:productId"
		},
		// usage
		trackUsage : {
			method : "POST",
			url : "controllers/shopping/usage", 
			isArray:false
		}		
	});
	


	function createOrGetExistingCart() {
		if (typeof localStorage.getItem("CART_ID") === "undefined" || localStorage.getItem("CART_ID") === null) {
			localStorage.setItem("CART_ID", "CART_ID_" + Math.floor((Math.random() * 10000) + 1));
		}
		return localStorage.getItem("CART_ID");
	}

	
	function trackUsage(viewAccessed) {
		return Cart.trackUsage({},{
			viewName : viewAccessed
		}).$promise;
	}

	function getCart() {
		return Cart.getCart({
			cartId : createOrGetExistingCart()
		}).$promise;
	}
	
	function getCartById(id) {
		return Cart.getCart({
			cartId : id
		}).$promise;
	}


	function updateCart(cart) {
		return Cart.updateCart(cart).$promise;
	}

	function addToCart(productId) {
		return Cart.addProductItem({
			cartId : createOrGetExistingCart(),
			productId : productId
		}).$promise.then(function(cartReturned) {
			messageSvc.displayMsg("Item added to cart ");
			console.log("Item added successfuly to cart ID " + cartReturned.cartId);
		})
		// deal with errors
		["catch"](function(error) {
			messageSvc.displayError(error);
		});
	}

	function removeAllProductItems(productId) {
		return Cart.removeAllProductItems({
			cartId : createOrGetExistingCart(),
			productId : productId
		}).$promise;
	}

	function calculateTotalCartCosts(cart) {
		var cartTotal = 0;

		try {
			if (cart != null) {
				if (cart.items != null && cart.items.length > 0) {
					for ( var i = 0; i < cart.items.length; i++) {

						// apply discount (this also dynamically adds a new property
						// called discount price to each game)
						cart.items[i] = pricingSvc.applyDiscount(cart.items[i]);

						// calculate cart total based on discount price
						cartTotal += cart.items[i].discountPrice;
					}

				}
				cart.total = cartTotal;
			} else {
				throw new Error("Cannot recalculate - Null cart data");
			}
		} catch (calcEx) {
			console.log("%c" + calcEx, "color: red");
			throw new Error("Error calculating cart");
		}
		return cart;
	}
	;

	function createCartForUpdate(cartFromView) {
		var cart = {
			cartId : cartFromView.cartId,
			totalCount : cartFromView.totalCount,
			items : new Array()
		};

		angular.forEach(cartFromView.items, function(item) {
			var pItem = {
				productId : item.productId,
				quantity : item.quantity
			};
			cart.items.push(pItem);
		});

		return cart;
	}

	return {
		getCart : getCart,
		createOrGetExistingCart : createOrGetExistingCart,
		addToCart : addToCart,
		updateCart : updateCart,
		removeAllProductItems : removeAllProductItems,
		calculateTotalCartCosts : calculateTotalCartCosts,
		createCartForUpdate : createCartForUpdate,
		getCartById : getCartById,
		trackUsage : trackUsage
	};

} ]);