package com.gamestore.controllers;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import com.gamestore.dataservices.DataService;
import com.gamestore.entities.ShoppingCart;
import com.gamestore.exceptions.ErrorMessage;

@RestController
@RequestMapping("/shopping")
public class ShoppingCartController {

	@Autowired
	private DataService dataService;

	@RequestMapping(value = "/carts", method = RequestMethod.POST)
	public ResponseEntity<ShoppingCart> createCart(HttpServletRequest request) throws Exception {

		ShoppingCart cart = dataService.getCart(null);

		return new ResponseEntity<ShoppingCart>(cart, HttpStatus.OK);
	}

	@RequestMapping(value = "/carts/{cartId}", method = RequestMethod.GET)
	public ResponseEntity<ShoppingCart> getCart(@PathVariable String cartId, HttpServletRequest request) throws Exception {

		ShoppingCart newCart = dataService.getCart(cartId);

		return new ResponseEntity<ShoppingCart>(newCart, HttpStatus.OK);
	}

	@RequestMapping(value = "/carts/{cartId}/products/{productId}", method = RequestMethod.POST)
	public ResponseEntity<ShoppingCart> addToCart(@PathVariable String cartId, @PathVariable String productId, HttpServletRequest request) throws Exception {

		ShoppingCart newCart = dataService.addToCart(cartId, productId);

		return new ResponseEntity<ShoppingCart>(newCart, HttpStatus.OK);
	}

	@RequestMapping(value = "/carts/{cartId}", method = RequestMethod.PUT)
	public ResponseEntity<ShoppingCart> updateCart(@RequestBody ShoppingCart cart, @PathVariable String cartId, HttpServletRequest request) throws Exception {

		ShoppingCart newCart = dataService.updateQuantities(cart);

		return new ResponseEntity<ShoppingCart>(newCart, HttpStatus.OK);
	}

	@RequestMapping(value = "/carts/{cartId}/products/{productId}", method = RequestMethod.DELETE)
	public ResponseEntity<ShoppingCart> deleteFromCart(@PathVariable String cartId, @PathVariable String productId, HttpServletRequest request) throws Exception {

		ShoppingCart newCart = dataService.deleteFromCart(cartId, productId);

		return new ResponseEntity<ShoppingCart>(newCart, HttpStatus.OK);
	}

	@ExceptionHandler(Exception.class)
	@ResponseStatus(value = HttpStatus.BAD_REQUEST)
	public ErrorMessage handleGeneralException(Exception e) {
		return new ErrorMessage("Unable to complete user request.", e.getMessage());
	}

}