package com.gamestore.dataservices;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import com.gamestore.entities.Game;
import com.gamestore.entities.ShoppingCart;

@Service
public class DataServiceImpl implements DataService {

	// cart
	private static HashMap<String, ShoppingCart> carts = new HashMap<String, ShoppingCart>();

	// inventory
	private static List<Game> list = new ArrayList<Game>() {

		private static final long serialVersionUID = 1L;

		{
			add(new Game("Madden NFL 15", "madden_nfl_15",
					"Madden NFL 15 has an all-new broadcast-style presentation, authentic NFL jumbotrons, Player Spotlight Montages, as well as new pregame and halftime shows.",
					"App/images/madden15.png", 54.95, 0));

			add(new Game("Far Cry 4", "far_cray_4",
					"Building on its award-winning predecessor, Far Cry 4 delivers its best experience ever! Experience an entirely new integrated co-op play in the incredibly open worled of Kyrat.",
					"App/images/farcry.png", 59.96, 0));

			add(new Game(
					"Assassin's Creed Unity",
					"acreed_unity",
					"Experience the cobblestone streets of Paris in 1789 during the French revolution. Rise up with the commoners against an oppressive aristocracy. In this heroic journey, Arno tries to find out who's really behind the Revolution. His pursuit leads him into a struggle for the fate of a nation - one that will transform him into a true Master Assassin.",
					"App/images/creed.png", 53.83, 0));

			add(new Game(
					"Call of Duty Advanced Warfare",
					"cod_adv_war",
					"Join the battle of the future in a new era of combat for the Call of Duty franchise. Academy Award winning actor Kevin Spacey stars as Jonathan Irons who's bent on reshaping future of war.",
					"App/images/callOfDuty.png", 59.00, 0));

			add(new Game(
					"Middle Earth: Shadow of Mordor",
					"mid_earth_shadow_mordor",
					"As Talion, a Ranger of the Black Gate, you'll travel through brutal combos while mastering powerful new skills and weaponry in an attempt to become the most feared force in Mordor in this latest Lord of the Rings saga.",
					"App/images/mordor.png", 49.78, 0));

		}
	};

	public List<Game> findAll() {
		return DataServiceImpl.list;
	}

	@Override
	public List<Game> findMatch(String srchTerm) {
		List<Game> results = new ArrayList<Game>();

		if (!StringUtils.isBlank(srchTerm)) {
			for (Game game : DataServiceImpl.list) {
				if (StringUtils.containsIgnoreCase(game.getName(), srchTerm)) {
					results.add(game);
				}
			}
		}

		return results;
	}

	@Override
	public Game findById(String id) {
		Game gameFound = null;

		if (!StringUtils.isBlank(id)) {
			for (Game game : DataServiceImpl.list) {
				if (game.getProductId().equals(id)) {
					gameFound = game;
					break;
				}
			}
		}

		return gameFound;
	}

	@Override
	public ShoppingCart getCart(String cartId) throws Exception {
		ShoppingCart cart = DataServiceImpl.carts.get(cartId);

		if (cart == null) {
			cart = new ShoppingCart();
			cart.setCartId(cartId);
			cart = updateUserCarts(cart);
		}

		return cart;
	}

	@Override
	public ShoppingCart addToCart(String cartid, String productId) throws Exception {

		// get shopping cart requested
		ShoppingCart cart = getCart(cartid);

		// update it with this product
		ShoppingCart updatedCart = addOrUpdateCart(productId, cart);

		// return updated version to controller
		return updatedCart;
	}

	@Override
	public ShoppingCart deleteFromCart(String cartid, String productId) throws Exception {

		// get shopping cart requested
		ShoppingCart cart = getCart(cartid);

		// update it with this product
		deleteItem(productId, cart);

		// update total
		cart.setTotalCount(getTotalItemCount(cart));

		// return updated version to controller
		return updateUserCarts(cart);
	}

	@Override
	public ShoppingCart updateQuantities(ShoppingCart cart) throws Exception {

		List<Game> updatedItemList = null;

		if (cart.getItems() != null && cart.getItems().size() > 0) {

			updatedItemList = new ArrayList<Game>();

			for (int i = 0; i < cart.getItems().size(); i++) {
				Game currentGame = cart.getItems().get(i);
				if (currentGame != null && currentGame.getQuantity() != 0) {
					Game gameFromInventory = findById(currentGame.getProductId());
					gameFromInventory.setQuantity(currentGame.getQuantity());
					updatedItemList.add(gameFromInventory);
				}
			}

			cart.setItems(updatedItemList);
		}

		// update total
		cart.setTotalCount(getTotalItemCount(cart));

		return updateUserCarts(cart);
	}

	private ShoppingCart addOrUpdateCart(String productId, ShoppingCart cart) {

		// start an item list in cart if there's not one already
		if (cart.getItems() == null) {
			cart.setItems(new ArrayList<Game>());
		}

		// item added already in cart? if so, just increment quantity
		boolean itemUpdated = updateExistingItem(productId, cart);

		// if no items or no match, get game from inventory and add it
		if (!itemUpdated) {
			addItemToCart(productId, cart);
		}

		// update total
		cart.setTotalCount(getTotalItemCount(cart));

		// put the updated version of the cart in the list of user carts
		return updateUserCarts(cart);
	}

	private void deleteItem(String productId, ShoppingCart cart) {
		if (cart.getItems().size() > 0) {
			for (int i = 0; i < cart.getItems().size(); i++) {
				Game currentGame = cart.getItems().get(i);
				if (currentGame.getProductId().equals(productId)) {
					cart.getItems().remove(i);
					break;
				}
			}
		}

	}

	private int getTotalItemCount(ShoppingCart cart) {
		int count = 0;
		List<Game> items = cart.getItems();
		if (items != null && items.size() > 0) {
			for (Game game : items) {
				if (game != null) {
					count = count + game.getQuantity();
				}
			}
		}
		return count;
	}

	private void addItemToCart(String productId, ShoppingCart cart) {
		Game game = findById(productId);
		if (game != null) {
			game.setQuantity(1);
			cart.getItems().add(game);
		}
	}

	private boolean updateExistingItem(String productId, ShoppingCart cart) {
		boolean matchingItemFound = false;

		if (cart.getItems().size() > 0) {
			for (int i = 0; i < cart.getItems().size(); i++) {
				Game currentGame = cart.getItems().get(i);
				if (currentGame.getProductId().equals(productId)) {
					currentGame.setQuantity(currentGame.getQuantity() + 1);
					cart.getItems().set(i, currentGame);
					matchingItemFound = true;
					break;
				}
			}
		}

		return matchingItemFound;
	}

	private ShoppingCart updateUserCarts(ShoppingCart cart) {
		DataServiceImpl.carts.put(cart.getCartId(), cart);
		return DataServiceImpl.carts.get(cart.getCartId());
	}

}
